﻿using System;
using System.Collections;
using System.Reflection;
using HeuristicLab.Persistence.Core;
using HeuristicLab.Persistence.Interfaces;
using System.Collections.Generic;

namespace HeuristicLab.Persistence.Default.Decomposers {
  
  public class EnumerableDecomposer : IDecomposer {

    public bool CanDecompose(Type type) {
      return
        type.GetInterface(typeof(IEnumerable).FullName) != null &&
        type.GetMethod("Add") != null &&
        type.GetMethod("Add").GetParameters().Length == 1 &&
        type.GetConstructor(
          BindingFlags.Public |
          BindingFlags.NonPublic |
          BindingFlags.Instance,
          null, Type.EmptyTypes, null) != null;        
    }

    public IEnumerable<Tag> DeCompose(object obj) {
      foreach (object o in (IEnumerable)obj) {
        yield return new Tag(null, o);
      }
    }

    public object CreateInstance(Type type) {
      return Activator.CreateInstance(type, true);      
    }

    public object Populate(object instance, IEnumerable<Tag> objects, Type type) {
      MethodInfo addMethod = type.GetMethod("Add");
      foreach (var pair in objects) {
        addMethod.Invoke(instance, new[] {pair.Value});
      }
      return instance;
    }

  }

}
